import Tkinter
import sys
import ToolTip

class Control(Tkinter.Frame):
    def __init__(self, master, integer=1, step=1, value=0.0, min=-sys.maxint, max=sys.maxint,\
                 label='', labelside='w', fg='black', variable=None, command=None, tooltip=None,\
                 activebackground=None, button_fg='black', entry_fg=None, entry_bg=None,\
                 selectforeground=None, selectbackground=None, *args, **kw):
        Tkinter.Frame.__init__(self, master, *args, **kw)
        
        # define keys the widget will respond to, all others will be ignored
        self._keys = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'Return', 'Tab', 'KP_Down', 'KP_Up',\
                      'KP_Left', 'KP_Right', 'KP_End', 'KP_Begin', 'KP_Home', 'KP_Next', 'KP_Prior', 'KP_Insert',\
                      'KP_Enter', 'Up', 'Down', 'ISO_Left_Tab', 'BackSpace', 'Delete', 'Left', 'Right']
        if not integer:
            for k in ('period', 'comma', 'KP_Delete'):
                self._keys.append(k)
        
        self._integer = integer
        self._step = step
        self._max = max
        self._min = min
        self._command = command
        
        if not variable:
            if integer:
                self._var = Tkinter.IntVar()
            else:
                self._var = Tkinter.DoubleVar()
        else:
            self._var = variable
        self._textvar = Tkinter.StringVar()

        grid_opts = {'w':(0, 0, 0, 1), 'e':(0, 1, 0, 0), 'n':(0, 0, 1, 0), 's':(1, 0, 0, 0)}
        #7x4 bitmaps, stolen from tix
        arrow_up = '#define incr_width 7\n#define incr_height 4\nstatic char incr_bits[] = {   0x08, 0x1c, 0x3e, 0x7f};'
        arrow_down = '#define decr_width 7\n#define decr_height 4\nstatic char decr_bits[] = {   0x7f, 0x3e, 0x1c, 0x08};'
        self._arrow_up = Tkinter.BitmapImage(data=arrow_up, foreground=button_fg)
        self._arrow_down = Tkinter.BitmapImage(data=arrow_down, foreground=button_fg)

        self.label = Tkinter.Label(self, text=label, bg=self['bg'], fg=fg)
        self.label.grid(row=grid_opts[labelside][0], column=grid_opts[labelside][1])

        self._frame = Tkinter.Frame(self, relief='groove', bd=2, bg=self['bg'])
        self._frame.grid(row=grid_opts[labelside][2], column=grid_opts[labelside][3])
        self._frame.grid_rowconfigure(0, weight=1)
        self.entry = Tkinter.Entry(self._frame, width=4, textvariable=self._textvar,\
                                   highlightbackground=self['bg'], fg=entry_fg, bg=entry_bg,\
                                   selectforeground=selectforeground, selectbackground=selectbackground, exportselection=0)
        self.entry.grid(row=0, column=0)
        self._colors = {'bg':self.entry['bg'], 'fg':self.entry['fg'], 'selectbg':self.entry['selectbackground'],\
                            'selectfg':self.entry['selectforeground']}
        # validate='all' isn't good enough here:
        self.entry.bind('<Any-KeyPress>', self._check_key)
        self.entry.bind('<FocusOut>', lambda event: self.set())
        self.entry.bind('<Return>', lambda event: self.entry.event_generate('<Tab>'))
        self.entry.bind('<KP_Enter>', lambda event: self.entry.event_generate('<Tab>'))
        self.entry.bind('<Up>', self._increase)
        self.entry.bind('<Down>', self._decrease)
        
        self._bframe = Tkinter.Frame(self._frame, bg=self['bg'])
        self._bframe.grid(row=0, column=1, sticky='ns')
        self.up = Tkinter.Button(self._bframe, image=self._arrow_up, pady=0, padx=0,\
                                 relief='flat', bd=1, takefocus=0, bg=self['bg'],\
                                 activebackground=activebackground, highlightbackground=self['bg'])
        self.up.grid(row=0, column=0, sticky='n')
        self.down = Tkinter.Button(self._bframe, image=self._arrow_down, pady=0, padx=0,\
                                   relief='flat', bd=1, takefocus=0, bg=self['bg'],\
                                   activebackground=activebackground, highlightbackground=self['bg'])
        self.down.grid(row=1, column=0, sticky='s')

        #some stuff we need for repeated in-/decreasing the value while B1 is pressed
        self._mousepress = 0
        for button in (self.up, self.down):
            button.bind('<1>', self._start_buttons)
            button.bind('<Any-ButtonRelease-1>', self._stop_buttons)
            button.bind('<Leave>', self._stop_buttons)
            
        self.set(value)
        
        if tooltip:
            self.tooltip = ToolTip.ToolTip(self, tooltip)
##------grrr... Entry widget misses the disabledforeground option, a little hack to fix this---##

    def disable(self):
        self.up.configure(state='disabled')
        self.down.configure(state='disabled')
        self.entry.selection_clear()
        self.entry.configure(state='disabled', fg='gray', selectbackground=self._colors['bg'],\
                             selectforeground='gray')

    def enable(self):
        self.up.configure(state='normal')
        self.down.configure(state='normal')
        self.entry.configure(state='normal', fg=self._colors['fg'], selectbackground=self._colors['selectbg'],\
                             selectforeground=self._colors['selectfg'])

##-------main methods to use from outside of the widget-----------------------------##

    def get(self):
        return self._var.get()
       
    def set(self, value=None):
        self.entry.select_clear()
        old = self._var.get()
        value = self._validate(value)
        if value == 'False':
            # test failed, restore the old value
            self._textvar.set(str(old))
        else:
            self._var.set(value)
            self._textvar.set(str(value))
        if self._var.get() != old:
            # execute command() only if the value has actually changed
            if self._command:
                self._command()

##-----------validation stuff--------------------------------------------------------##

    def _check_key(self, event):
        if not event.keysym in self._keys:
            return 'break'
    
    def _validate(self, value=None):
        if value == None:
            value = self._textvar.get()
        if value == self._textvar.get():
            # allow the use of commata instead of periods
            value = value.replace(',', '.')
        try:
            if self._integer:
                value = (int(value))
            else:
                value = (float(value))
        except:
            return 'False'
        if value > self._max:
            value = self._max
        elif value < self._min:
            value = self._min
        return value

##-----methods to increase/decrease the widget's value with Up/Down keys--------------##

    def _increase(self, event=None):
        if self.entry['state'] != 'disabled':
            # in case something was just typed in manually:
            self.set()
            old = self._var.get()
            new = old + self._step
            self.set(new)
    
    def _decrease(self, event=None):
        if self.entry['state'] != 'disabled':
            # in case something was just typed in manually:
            self.set()
            old = self._var.get()
            new = old - self._step
            self.set(new)

##----------methods to control the behavior of the "up" and "down" buttons------------##

    def _button_increase(self):
        if self._mousepress:
            self._increase()
            self.after(50, self._button_increase)
             
    def _button_decrease(self):
        if self._mousepress:
            self._decrease()
            self.after(50, self._button_decrease)

    def _start_buttons(self, event):
        if self.entry['state'] != 'disabled':
            self.entry.focus_set()
            self._mousepress = 1
            if event.widget == self.up:
                self._increase()
                self.after(500, self._button_increase)
            elif event.widget == self.down:
                self._decrease()
                self.after(500, self._button_decrease)

    def _stop_buttons(self, event):
        self._mousepress = 0
