#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

#################################################################################
# scsiaddgui - a gui for the scsiadd utility
# copyright (c) 2003 - 2004 Michael Lange <klappnase@8ung.at>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the #hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#################################################################################

from Tkinter import *
from Control import Control
from ToolTip import ToolTip
from ScrolledText import ScrolledText
import sys
import tkMessageBox
import commands
import fileinput
import os
import locale
import gettext

programPath = os.path.abspath(sys.path[0])

# check where to look for translations
localedir = '/usr/share/locale'
if programPath.startswith('/usr/local'):
    localedir = '/usr/local/share/locale'

gettext.install('scsiaddgui', localedir)

class ReadonlyScrolledText(ScrolledText):
    def __init__(self, master, **kw):
        ScrolledText.__init__(self, master, cursor='', wrap='word', padx=0, pady=4, takefocus=0, **kw)
        self.frame.configure(relief='groove', bd=2, highlightthickness=1, highlightbackground=self.frame['bg'])
        self.vbar.configure(highlightthickness=0)
        self.vbar.bind('<FocusIn>', lambda event: self.frame.configure(highlightbackground=self.frame['highlightcolor']))
        self.vbar.bind('<FocusOut>', lambda event: self.frame.configure(highlightbackground=self.frame['bg']))
    def focus(self):
        self.vbar.focus_set()

class HelpDialog(Toplevel):
    def __init__(self, master, **kw):
        Toplevel.__init__(self, master, **kw)
        self.protocol('WM_DELETE_WINDOW', self.close)
        self.title(_('ScsiaddGui - Help'))
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)
        self.master = master
        
        self.text = ReadonlyScrolledText(self, bg="wheat")
        self.text.grid(row=0, column=0, sticky='news', padx=3, pady=3)
        helpfile = self.get_helpfile()
        for line in fileinput.input(helpfile):
            self.text.insert('end', line)
        self.text.config(state='disabled')
        self.button = Button(self, text=_('Close'), command=self.close)
        self.button.grid(row=1, column=0, pady=5)
    
    def get_helpfile(self):
        try:
            lang = locale.getdefaultlocale()[0][:2]
        except TypeError:
            # no language found, fall back to english
            lang = 'en'
        helpfile = os.path.join(programPath, 'help_'+lang)
        if not (os.path.exists(helpfile) and os.access(helpfile, os.R_OK)):
            return os.path.join(programPath, 'help_en')
        return helpfile

    def close(self, event=None):
        self.master.help_dialog = None
        self.destroy()

class OptConf:
    def __init__(self):
        self.opts = {'host':0, 'chan':0, 'i_d':0, 'lun':0}
        self.conffile = os.path.join(os.getenv('HOME'), '.scsiaddgui')

    def read_conffile(self):
        try:
            for line in fileinput.input(self.conffile):
                try:
                    l = line.split('=')
                    if self.opts.has_key(l[0]):
                        self.opts[l[0]] = int(l[1])
                except:
                    pass
        except:
            pass

    def write_conffile(self):
        f = open(self.conffile, 'w')
        for key in self.opts:
            f.write(key + '=' + str(self.opts[key]) + '\n')
        f.close()

        
class Mainwindow(Frame):
    def __init__(self, master, **kw):
        Frame.__init__(self, master, **kw)
        self.grid_rowconfigure(1, weight=1)
        for i in range(0, 4):
            self.grid_columnconfigure(i, weight=1)

        self.master = master
        self.help_dialog = None
        self.conf = OptConf()
        self.conf.read_conffile()
        
        self.host = IntVar()
        self.host.set(self.conf.opts['host'])
        self.chan = IntVar()
        self.chan.set(self.conf.opts['chan'])
        self.i_d = IntVar()
        self.i_d.set(self.conf.opts['i_d'])
        self.lun = IntVar()
        self.lun.set(self.conf.opts['lun'])
        
        self.label = Label(self, text=_('Attached devices:'))
        self.label.grid(row=0, column=0, columnspan=4, pady=4)
        self.display = ReadonlyScrolledText(self, width=50, height=15, state='disabled')
        self.display.grid(row=1, column=0, columnspan=4, sticky='news', padx=4)
        self.display.tag_config('yellow', lmargin1=5, rmargin=5)
        self.display.tag_config('khaki', background='khaki', lmargin1=5, rmargin=5)
        self.display.tag_raise('sel')
        
        self.hostselect = Control(self, min=0, max=7, label=_('Host:'), variable=self.host, value=self.host.get(),
                                  tooltip=_('Select the SCSI adapter card; 0 means the first card, 1 the second and so on'))
        self.hostselect.grid(row=2, column=0, pady=4)
        self.chanselect = Control(self, min=0, max=7, label=_('Channel:'), variable=self.chan, value=self.chan.get(),
                                  tooltip=_('Select the bus-no. of your adapter card (most cards only support bus 0)'))
        self.chanselect.grid(row=2, column=1, pady=4)
        self.idselect = Control(self, min=0, max=15, label=_('Id:'), variable=self.i_d, value=self.i_d.get(),
                                tooltip=_('Select the ID-no. of your device'))
        self.idselect.grid(row=2, column=2, pady=4)
        self.lunselect = Control(self, min=0, max=255, label=_('Lun:'), variable=self.lun, value=self.lun.get(),
                                 tooltip=_("Select the Logical Unit Number if you have a device that supports multiple LUN's"))
        self.lunselect.grid(row=2, column=3, pady=4)
        
        self.bframe = Frame(self)
        self.bframe.grid(row=3, column=0, columnspan=4, pady=4, sticky='ew')
        for c in range(3):
            self.bframe.grid_columnconfigure(c, weight=1)
        self.scanbutton = Button(self.bframe, text=_('Scan for devices'), width=16, command=lambda : self.run('scan'))
        self.scanbutton.grid(row=0, column=0, padx=8)
        ToolTip(self.scanbutton, _('Try to find all devices connected to the selected SCSI Bus/Channel'))
        self.addbutton = Button(self.bframe, text=_('Add device'), width=16, command=lambda : self.run('add'))
        self.addbutton.grid(row=0, column=1, padx=8)
        ToolTip(self.addbutton, _('Add the selected device to the system.'))
        self.removebutton = Button(self.bframe, text=_('Remove device'), width=16, command=lambda : self.run('remove'))
        self.removebutton.grid(row=0, column=2, padx=8)
        ToolTip(self.removebutton, _('Remove the selected device from the system'))
    
    def find_scsiadd(self):
        # check if scsiadd is available
        paths = os.getenv('PATH').split(':')
        for p in ('/sbin', '/usr/sbin', '/usr/local/sbin'):
            if not p in paths:
                paths.append(p)
        for p in paths:
            fname = os.path.join(p, 'scsiadd')
            if os.path.isfile(fname):
                if os.access(fname, os.X_OK):
                    return fname
                else:
                    return 'permission denied'
        return ''
        
    def non_root_box(self):
        tkMessageBox.showerror(title=_('ScsiaddGui'),
                               message=_('Unable to change the hardware configuration: permission denied.'))
    
    def not_found_box(self):
        tkMessageBox.showerror(title=_('ScsiaddGui'),
                               message=_('Cannot find "scsiadd". Please check your installation.'))
        
    def run(self, action):
        if action == 'print':
            return self.update_display(commands.getoutput('cat /proc/scsi/scsi'))
        # scsiadd returns 0 even if we don't have permission to change
        # the hardware configuration, so we'll have to check if we are root ourselves
        if os.getuid():
            return self.non_root_box()
        scsiadd = self.find_scsiadd()
        if scsiadd == 'permission denied':
            return self.non_root_box()
        elif not scsiadd:
            return self.not_found_box()
        # all tests passed
        oldcursor = self.master.cget('cursor')
        if action == 'scan':
            self.master.configure(cursor='watch')
            self.master.update_idletasks()
            cmd = ' '.join((scsiadd, '-s', str(self.host.get()), str(self.chan.get())))
        elif action == 'add':
            cmd =' '.join((scsiadd, '-a', str(self.host.get()), str(self.chan.get()),\
                           str(self.i_d.get()), str(self.lun.get())))
        else:
            cmd = ' '.join((scsiadd, '-r', str(self.host.get()), str(self.chan.get()),\
                           str(self.i_d.get()), str(self.lun.get())))
        self.update_display(commands.getoutput(cmd))
        self.master.configure(cursor=oldcursor)
                
    def format_msg(self, msg):
        if msg[-1] != '\n':
            msg = msg + '\n'
        msg = msg.replace('Vendor:', _('Vendor:'))
        msg = msg.replace('Model:', _('Model:'))
        msg = msg.replace('Type:', _('Type:'))
        msg = msg.replace('Host:', _('Host:'))
        msg = msg.replace('Channel:', _('Channel:'))
        msg = msg.replace('Id:', _('Id:'))
        msg = msg.replace('Lun:', _('Lun:'))
        return msg
        
    def toggle_tag(self, tag):
        if tag == 'yellow':
            return 'khaki'
        return 'yellow'
    
    def update_display(self, message):
        # split the output message into a list of devices
        # quick'n'dirty: use a german special character as separator
        message = message.replace('Host:', '\xdfHost:')
        if '\xdf' in message:
            # one or more devices were found
            dlist = message.split('\xdf')[1:]
        else:
            # no devices found
            dlist = [_('No SCSI devices found.')]
        self.display['state'] = 'normal'
        self.display.delete(0.0, 'end')
        tag = 'yellow'
        for item in dlist:
            msg = self.format_msg(item)
            self.display.insert('end', msg, tag)
            tag = self.toggle_tag(tag)
        self.display['state'] = 'disabled'

    def show_help(self):
        if self.help_dialog:
            self.help_dialog.tkraise()
            self.help_dialog.deiconify()
        else:
            self.help_dialog = HelpDialog(self, relief='ridge', bd=3)
        self.help_dialog.text.focus()

    def close(self):
        self.conf.opts['host'] = self.host.get()
        self.conf.opts['chan'] = self.chan.get()
        self.conf.opts['i_d'] = self.i_d.get()
        self.conf.opts['lun'] = self.lun.get()
        self.conf.write_conffile()
        self.master.quit()
    

def infobox():
    tkMessageBox.showinfo(title=_('About ScsiaddGui'),\
                          message=_('ScsiaddGui - by Michael Lange\nContact: <klappnase@8ung.at>'))

def main():
    root = Tk()
    root.option_readfile(os.path.join(programPath,'optionDB'))
    root.title(string=_('ScsiaddGui'))
    root.bind_class('Button', '<Return>', lambda event :root.event_generate('<space>'))
    mw = Mainwindow(root, relief='ridge', bd=3)
    mw.pack(fill='both', expand=1)
    mw.run('print')
    root.protocol('WM_DELETE_WINDOW', mw.close)
    menubar = Menu(root)
    progm = Menu(menubar)
    progm.add_separator()
    progm.add_command(label=_('Quit'), command=mw.close)
    menubar.add_cascade(label=_('Program'), underline=0, menu=progm)
    helpm = Menu(menubar)
    helpm.add_separator()
    helpm.add_command(label=_('Help...'), command=mw.show_help)
    helpm.add_command(label=_('About...'), command=infobox)
    menubar.add_cascade(label=_('Help'), underline=0, menu=helpm)
    root.config(menu=menubar)
    root.mainloop()
    root.destroy()
        
if __name__ == "__main__":
    main()
